/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2017 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.executable;

import static org.fortiss.af3.expression.utils.ExpressionModelElementFactory.funcCall;
import static org.fortiss.af3.expression.utils.ExpressionModelElementFactory.intConst;

import org.fortiss.af3.component.model.OutputPort;
import org.fortiss.af3.expression.model.terms.IExpressionTerm;
import org.fortiss.af3.platform.generic.generator.executable.GenericTransmitterExecutable;
import org.fortiss.af3.platform.model.annotation.GPIOPin;
import org.fortiss.af3.platform.model.generic.GenericTransmitter;
import org.fortiss.af3.platform.raspberry.model.GPO;
import org.fortiss.tooling.base.utils.AnnotationUtils;

/**
 * Executable for {@link GPO}.
 * 
 * @author eder
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash:
 */
public class GPOExecutable extends GenericTransmitterExecutable {

	/** Constructor. */
	public GPOExecutable(GenericTransmitter modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getInitialization() {
		return funcCall("gpio_init", intConst(getPinFromModelElement(modelElement)));
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getNoValWriteAccessor(OutputPort logicalSignal) {
		return funcCall("gpio_set_noval", intConst(getPinFromModelElement(modelElement)));
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getTermination() {
		return funcCall("gpio_term", intConst(getPinFromModelElement(modelElement)));
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getValueWriteAccessor(OutputPort logicalSignal, IExpressionTerm value) {
		return funcCall("gpio_write", intConst(getPinFromModelElement(modelElement)), value);
	}

	private static int getPinFromModelElement(GenericTransmitter modelElement) {
		return AnnotationUtils.getAnnotation(modelElement, GPIOPin.class).getPinNumber();
	}
}
